local _, RS = ...

SLASH_REACTIONS1 = "/reactions"
SLASH_REACTIONS2 = "/rs"
function SlashCmdList.REACTIONS(input)
	input = input or ""

	local command, value = input:match("(%w+)%s*(.*)%s*$")
	command = command or input
	command = command:lower()

	-- check if it's a test message now so that value can be changed to lowercase for other options
	if command == "testmessage" then
		RS.Reactions.AttemptReaction(nil, 0, nil, UnitName("player"), nil, RS.Common.ActionType.HIT, true, value)
		return
	elseif command == "use" then
		-- the spell name may have quotation marks around it (and must if it's a multiple word spell)
		-- /rs use "Multiple Word Spell" [target1] [target2]
		-- /rs use SingleWordSpell [target1] [target2]
		local spell, target
		if value:find("^[\"']") then
			spell, target = value:match("^[\"'](.-)[\"']%s*(%S*)")
		else
			spell, target = value:match("^(%a+)%s*(%S*)")
		end
		if spell then
			if target == "%t" then
				target = (UnitName("target"))
			end
			RS.Reactions.AttemptReaction(spell, 0, nil, nil, target, RS.Common.ActionType.HIT)
		end
		return
	end
	if value then
		value = value:lower()
	end

	if command == "" then
		if not IsAddOnLoaded("Reactions_Settings") then
			local loaded, reason = LoadAddOn("Reactions_Settings")
			if not loaded then
				DEFAULT_CHAT_FRAME:AddMessage("Unable to load the Reactions_Settings addon: " .. (reason or "no error message given!"), 1, 0, 0)
				return
			end
			RSGUI:Create(ReactionsSave)
		end
		RSGUI:Show()
	--------------------------------------------------
	-- /reactions on
	--------------------------------------------------
	elseif command == "on" then
		ReactionsSave.enabled = true
		RS.Events:RegisterReactionsEvents()
		if value ~= "quiet" then
			DEFAULT_CHAT_FRAME:AddMessage("Reactions are now enabled.")
		end
		if RSGUI then
			RSGUI:UpdateWidgets()
		end
	--------------------------------------------------
	-- /reactions off
	--------------------------------------------------
	elseif command == "off" then
		ReactionsSave.enabled = false
		RS.Events:UnregisterReactionsEvents()
		RS.Queue:CancelAll()
		if value ~= "quiet" then
			DEFAULT_CHAT_FRAME:AddMessage("Reactions are now disabled.")
		end
		if RSGUI then
			RSGUI:UpdateWidgets()
		end
	--------------------------------------------------
	-- /reactions test
	--------------------------------------------------
	elseif command == "test" then
		value = lower(value)
		if value == "on" then
			ReactionsSave.testing = true
			DEFAULT_CHAT_FRAME:AddMessage("Test mode is now enabled (only you will see messages said).")
		elseif value == "off" then
			ReactionsSave.testing = false
			DEFAULT_CHAT_FRAME:AddMessage("Test mode is now disabled.")
		else
			DEFAULT_CHAT_FRAME:AddMessage('Syntax: /reactions test <"on"|"off">')
		end
		if RSGUI then
			RSGUI:UpdateWidgets()
		end
	--------------------------------------------------
	-- /reactions mod
	--------------------------------------------------
	elseif (command == "chance" or command == "mod") and value and value ~= "" then
		local mod = tonumber(value)
		if mod and mod > 0 and mod <= 99999 then
			ReactionsSave.chanceMultiplier = mod
			DEFAULT_CHAT_FRAME:AddMessage("The chance multiplier has been set to: " .. ReactionsSave.chanceMultiplier)
		else
			DEFAULT_CHAT_FRAME:AddMessage("The multiplier value must be a number above 0 up to 99,999.")
		end
		if RSGUI then
			RSGUI:UpdateWidgets()
		end
	--------------------------------------------------
	-- /reactions cooldown
	--------------------------------------------------
	elseif command == "cooldown" and value and value ~= "" then
		local seconds = tonumber(value)
		if seconds and seconds >= 0 then
			ReactionsSave.globalCooldown = seconds
			DEFAULT_CHAT_FRAME:AddMessage("The global cooldown time between each reaction is now " .. seconds .. ".")
			if RSGUI then
				RSGUI:UpdateWidgets()
			end
		else
			DEFAULT_CHAT_FRAME:AddMessage("The wait value must be a number 0 or above.")
		end
	--------------------------------------------------
	-- /reactions shout
	--------------------------------------------------
	elseif command == "shout" then
		value = lower(value)
		if value == "on" then
			ReactionsSave.shoutMode = true
			DEFAULT_CHAT_FRAME:AddMessage("Shout mode is now enabled.")
		elseif value == "off" then
			ReactionsSave.shoutMode = false
			DEFAULT_CHAT_FRAME:AddMessage("Shout mode is now disabled.")
		else
			DEFAULT_CHAT_FRAME:AddMessage('Syntax: /reactions shout <"on"|"off">')
		end
		if RSGUI then
			RSGUI:UpdateWidgets()
		end
	--------------------------------------------------
	-- /reactions group <name> on/off
	--------------------------------------------------
	elseif command == "group" and value and value ~= "" then
		local group, setting = value:match("(.+) (%a+)$")
		if setting and (setting == "on" or setting == "off") then
			local groupList = ReactionsSave.groupList
			for name in pairs(groupList) do
				if name:lower() == group then
					groupList[name] = setting == "on" and true or false
					DEFAULT_CHAT_FRAME:AddMessage("The " .. name .. " group in now " .. setting .. ".")
					return
				end
			end
			DEFAULT_CHAT_FRAME:AddMessage("The group '" .. group .. "' doesn't exist.")
		else
			DEFAULT_CHAT_FRAME:AddMessage('Syntax: /reactions group <name> <"on"|"off">')
		end
	--------------------------------------------------
	-- hidden - used from GUI
	--------------------------------------------------
	elseif command == "updateautomatic" then -- automatic event time changed, so set a new time for the next one
		RS.Events:SetNextAutomaticTime()
	--------------------------------------------------
	-- bod command - show syntax
	--------------------------------------------------
	else
		DEFAULT_CHAT_FRAME:AddMessage('Reactions commands:', 1, 1, 0)
		DEFAULT_CHAT_FRAME:AddMessage('/rs |cffffff00(will open settings window)|r')
		DEFAULT_CHAT_FRAME:AddMessage('/rs <"on"|"off">')
		DEFAULT_CHAT_FRAME:AddMessage('/rs test <"on"|"off">')
		DEFAULT_CHAT_FRAME:AddMessage('/rs cooldown <seconds>')
		DEFAULT_CHAT_FRAME:AddMessage('/rs chance <multiplier>')
		DEFAULT_CHAT_FRAME:AddMessage('/rs shout <"on"|"off">')
		DEFAULT_CHAT_FRAME:AddMessage('/rs group <name> <"on"|"off">')
		DEFAULT_CHAT_FRAME:AddMessage('/rs use "<spell>" [target]')
		DEFAULT_CHAT_FRAME:AddMessage(" ")

		if not ReactionsSave.enabled then
			DEFAULT_CHAT_FRAME:AddMessage("The addon is currently disabled.")
		elseif ReactionsSave.testing then
			DEFAULT_CHAT_FRAME:AddMessage("The addon is currently in test mode.")
		end
		DEFAULT_CHAT_FRAME:AddMessage("The global cooldown for reacting is " .. ReactionsSave.globalCooldown .. " seconds.")
		if ReactionsSave.chanceMultiplier and ReactionsSave.chanceMultiplier ~= 1 then
			DEFAULT_CHAT_FRAME:AddMessage("The chance multiplier is set to: " .. ReactionsSave.chanceMultiplier .. "%")
		end
		if ReactionsSave.shoutMode then
			DEFAULT_CHAT_FRAME:AddMessage("SHOUT MODE IS ENABLED!", 1, 1, 0)
		end
	end
end
