local _, RS = ...
RS.Queue = {}
local Queue = RS.Queue

-- local references
local GetTime = GetTime
local table_remove = table.remove
local table_insert = table.insert

----------------------------------------------------------------------------------------------------
-- future reaction queue for reactions using <new #>
----------------------------------------------------------------------------------------------------
local futureReactionList = {} -- table of future reaction messages - see AddFutureReaction() for structure - soonest time left goes on end
local nextFutureReaction = 0 -- GetTime() of the soonest future message

Queue.updateFrame = CreateFrame("frame")
Queue.updateFrame:Hide() -- stops OnUpdate()
Queue.updateFrame:SetScript("OnUpdate", function(self)
	if GetTime() < nextFutureReaction then
		return
	end

	-- the shortest time starts at the end of the table, so go backwards
	local size = #futureReactionList
	for i=#futureReactionList,1,-1 do
		if GetTime() < futureReactionList[i][1] then
			break
		else
			RS.Messages.UseMessage(futureReactionList[i][2], futureReactionList[i][3], futureReactionList[i][4])
			futureReactionList[i] = nil
			size = size - 1
		end
	end

	-- set the next time to check the messages, or stop checking if there are no more
	if size == 0 then
		Queue.updateFrame:Hide() -- stops OnUpdate()
	else
		nextFutureReaction = futureReactionList[size][1]
	end
end)

-- add a message to the queue - the shortest time goes on the end
function Queue:AddFutureReaction(message, channel, language, seconds)
	local newMessage = {GetTime() + seconds, message, channel, language}

	local size = #futureReactionList
	local isSoonest = true

	if size == 0 then
		futureReactionList[1] = newMessage
	else
		local added
		for i=size,1,-1 do
			if newMessage[1] < futureReactionList[i][1] then
				table_insert(futureReactionList, i+1, newMessage)
				added = true
				break
			end
			isSoonest = false
		end
		if not added then
			table_insert(futureReactionList, 1, newMessage)
		end
	end

	if isSoonest then
		nextFutureReaction = newMessage[1]
		self.updateFrame:Show() -- starts OnUpdate()
	end
end

-- remove any actions that will try to whisper a certain name (probably because they ignored you)
function Queue:RemoveFutureWhispers(name)
	if not name then
		return
	end
	local lowercaseName = name:lower()

	local canceled = false
	local size = #futureReactionList
	local message, command, target
	for i=size,1,-1 do
		message = futureReactionList[i][2]:lower()
		if futureReactionList[i][3] == "chat command" then
			command, target = message:match("^(/%S+)%s+(%S+) ")
			if target and whisperCommands[command] and target:lower() == lowercaseName then
				table_remove(futureReactionList, i)
				size = size - 1
				canceled = true
			end
		end
	end
	if size == 0 then
		Queue.updateFrame:Hide() -- stops OnUpdate()
	else
		nextFutureReaction = futureReactionList[size][1]
	end
	if canceled then
		DEFAULT_CHAT_FRAME:AddMessage("|cffFFFFE0Reactions:|r Future whispers to " .. name .. " have been canceled.")
	end
end

-- cancel all queued actions
function Queue:CancelAll()
	self.updateFrame:Hide()
	futureReactionList = {}
end
